#!/usr/bin/python3
import sys
import re


# -- It was part of the v1.0 sasm.py
class Lexer(object):
    """General functions for parsing"""

    # - Symbol used for defining comments in the assembler
    # - It is a parameter. It can be changed
    # COMMENT_SYMBOL = "//"  # - C / Verilog style
    # COMMENT_SYMBOL = "#"  # - Python style
    COMMENT_SYMBOL = ";"

    # Symbols for representing hexadecimal numbers
    # SYM_HEX = "0x"  # - Notation in C / python style
    SYM_HEX = "H'"  # - Notation in Simlez

    # Regular expresions for Parsing Hexadecimal numbers
    # - In Simplez, hexadecimal numbers are written as: H'8A, H'960A, etc
    REGEX_HEX = r"^{}[0-9a-fA-F]+$".format(SYM_HEX.upper())

    @staticmethod
    def is_comment(word):
        """Return True if the word is a comment"""

        # - Split the word usign the COMMENT_SYMBOL
        words = word.split(Lexer.COMMENT_SYMBOL)

        # - It does not contains the comment symbol
        if (len(words) == 1):
            return False

        # - The first word should be null (no charcters before the
        # - comment symbol)
        if (len(words[0]) == 0):
            return True
        else:
            return False

    @staticmethod
    def is_comment_line(line):
        """Returns true if the whole line is a comment"""

        # -- Divide the line into a list of words
        words = line.split()

        # - It is a commnet line if the first words is a comment
        return Lexer.is_comment(words[0])

    @staticmethod
    def is_blank_line(line):
        """Returns true if the line is blank"""

        # -- Divide the line into a list of words
        words = line.split()

        # -- If it is a blank line, ignore it
        if len(words) == 0:
            return True
        else:
            return False

    @staticmethod
    def is_hexdigit(word):
        """Returns True if words is an ASCII hexadecimal number"""

        if re.search(Lexer.REGEX_HEX, word):
            return True
        else:
            return False

    @staticmethod
    def is_number(word):
        """Determine if the word is a number (in decimal) or in hexadecimal
           It returns:
           -True: is a number
           -False: is not a number"""
        return word.isdigit() or Lexer.is_hexdigit(word)


class SyntaxError(Exception):
    """Syntax error exceptions"""
    def __init__(self, msg, nline):
        self.msg = msg       # - Sintax error message
        self.nline = nline   # - Number of line were the sintax error is located


class Test_progs(object):
    """Program example 1: adding two numbes"""
    SIMPLEZ1 = [0x0, 0x205, 0x406, 0x007, 0xE00, 0x007, 0x008, 0x00]

    """Program example 2: Sum of the 10 term of the fibonacci serie"""
    SIMPLEZ2 = [0xA00, 0x02F, 0x233, 0x030, 0x032, 0x234, 0x02E, 0x22F, 0x430,
                0x031, 0x432, 0x032, 0x230, 0x02F, 0x231, 0x030, 0x22E, 0xC00,
                0x815, 0x02E, 0x607, 0x232, 0xE00,     0,     0,     0,     0,
                0,         0,     0,     0,     0,     0,     0,     0,     0,
                0,         0,     0,     0,     0,     0,     0,     0,     0,
                0,         0,     0,     0,     0,     0,     1,     8,     0,
                0,         0,     0,     0,     0,     0,
                ]


class simplez(object):
    """Simplez simulator"""

    NEMONIC = ["ST", "LD", "ADD", "BR", "BZ", "CLR", "DEC", "HALT"]
    OP_ST = 0
    OP_LD = 1
    OP_ADD = 2
    OP_BR = 3
    OP_BZ = 4
    OP_CLR = 5
    OP_DEC = 6
    OP_HALT = 7

    # -- Extended opcodes
    XOP_HALT = 0xE
    XOP_WAIT = 0xF

    # -- States for the processor
    INIT = 0  # - Initial state (no instruction executed yet)
    RUNNING = 1  # - Simulating
    STOPED = 2  # - Micro stopped. Halt instruction executed

    def __init__(self):
        self._A = 0   # - A register
        self._PC = 0  # - Program counter
        self._Z = 0   # - Z flag
        self._mem = [0 for i in range(512)]  # - Simplez memory
        self.state = self.INIT  # - Processor state
        self._vars = {}  # - Dictionary of selected variables to show
        self._load_addr = 0  # - Address for loading the machine code

    def _decode(self, inst):
        """Return the opcode and argument of a given instruction
           in machine code"""
        return (inst >> 9, inst & 0x1FF)

    def show(self):
        print("PC = H'{:03X}".format(self._PC))
        print("A = H'{:03X}".format(self._A))
        print("Z = {}".format(self._Z))
        self.vars()
        if self.state == self.STOPED:
            print("Micro stopped! Halt executed")
        else:
            # - Fetch the current instruction
            ir = self._mem[self._PC]

            # - Print the current instruction
            print("Next: [{:03X}] {}".format(self._PC, self._code2asm(ir)))

    def load(self, prog):
        """Load a program into the memory. The program consist of a
           list of consecutive instructions"""

        l = len(prog)
        self._mem[0:l] = prog

    def mem(self, nblocks=32):
        """Print the memory
           nblocks = number of blocks (of 16 words) to print"""
        print("       0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F")
        for block in range(nblocks):
            linecode = ""
            for pos in self._mem[(block * 16):(block*16 + 16)]:
                linecode += "{:03X} ".format(pos)

            print("[{:03X}] {}".format(block*16, linecode))

    def vars(self):
        """Show the current vars"""
        for addr, label in self._vars.items():
            # - Read the variable value from memory
            value = self._mem[addr]

            # - Show the variable
            print("[{0:03X}] {1} = H'{2:03X} ({2})".format(addr, label, value))

    def add_var(self, varname, varaddr):
        """Add the pair varname (string) and address
           varname: String with the given name to the variable
           varaddr: The address were the variable is stored
        """
        # -- Add the var to the dictionary
        # -- TODO: check for duplicates!
        self._vars[varaddr] = varname.upper()

    def list(self, ninst=10, addr_ini=0):
        """List the memory as assembly code
           ninst: Number of instructions to show
           addr_ini: Initial address
        """
        for addr in range(addr_ini, addr_ini + ninst):
            # - Fetch the instruction from memory
            inst = self._mem[addr]
            print("[{:03X}] {}".format(addr, self._code2asm(inst)))

    def _execute(self, opcode, addr, xopcode):
        """Execute the give instruction"""

        if opcode == self.OP_ST:
            self._mem[addr] = self._A

        elif opcode == self.OP_LD:
            self._A = self._mem[addr]

        elif opcode == self.OP_ADD:
            self._A += self._mem[addr]

        elif opcode == self.OP_CLR:
            self._A = 0

        elif opcode == self.OP_DEC:
            self._A -= 1

        elif opcode == self.OP_BR:
            self._PC = addr

        elif opcode == self.OP_BZ:
            if self._Z == 1:
                self._PC = addr

        elif opcode == self.OP_HALT:
            """Check the extended instructions"""
            print("HALT. xopcode: {}".format(xopcode))
            if xopcode == self.XOP_HALT:
                self.state = self.STOPED

    def _single_step(self):
        """Simulate only the next instruction"""
        # - Fetch the current instruction
        ir = self._mem[self._PC]

        # - Print the current instruction
        print("Executing: [{:03X}] {}".format(self._PC, self._code2asm(ir)))

        # - Point to the next instruction
        self._PC += 1

        # - Decode the instruction
        opcode, addr = self._decode(ir)

        # - Decode de extended instructions
        xopcode = ir >> 8

        # - Execute
        self._execute(opcode, addr, xopcode)

        # - Update the Z flag
        if self._A == 0:
            self._Z = 1
        else:
            self._Z = 0

    def run(self):
        """Simulate the program until the end """

        # -- Show info of the initial state
        if self.state == self.INIT:
            print("Initial state:")
            self.show()
            print()
            self.state = self.RUNNING

        # - Simulate the whole program
        while self.state != self.STOPED:
            self._single_step()

        # - Show the final resultas
        self.show()

    def step(self, nsteps=1):
        """Simulate the given steps"""

        if self.state == self.INIT:
            print("Initial state:")
            self.show()
            print()
            self.state = self.RUNNING

        # -- Simulate nsteps
        for i in range(nsteps):
            if self.state == self.STOPED:
                print("Micro stopped! Halt executed")
                return
            self._single_step()
            self.show()

    def _is_mcode_line(self, line):
        """Return true if it is sintactically correct machine code line"""

        words = line.split()

        # -- First word should be a hexadecimal digit
        if re.search("^[0-9a-fA-F]+", words[0]):

            # - The next word (if any) should be a comment
            if (len(words) > 1):
                if Lexer.is_comment(words[1]):
                    return True
                else:
                    # -- Not a comment. Is not a correct mcode line
                    return False

            return True
        else:
            return False

    def _is_addr_line(self, line):
        words = line.split()

        # -- First word should be a @ + hexadecimal digit
        if re.search("^@[0-9a-fA-F]+", words[0]):

            # - The next word (if any) should be a comment
            if (len(words) > 1):
                if Lexer.is_comment(words[1]):
                    return True
                else:
                    # -- Not a comment. Is not a correct mcode line
                    return False

            return True
        else:
            return False

    def _parse_mcode(self, line):
        words = line.split()

        # - If is a correct machine code (only sintax)
        if self._is_mcode_line(line):

            # - Get the machine code word
            mcode = int(words[0], 16)

            # - Write the machine code in the memory, in the current addr
            self._mem[self._load_addr] = mcode
            self._load_addr += 1

            # print("{:03X}".format(mcode))

        # - Check if is an address code (@xxx)
        elif self._is_addr_line(line):
            addr = int(words[0][1:], 16)
            self._load_addr = addr
            # print("@{:03X}".format(addr))

        else:
            msg = "\nSINTAX ERROR. Unkwown line:\n{}".format(line)
            raise SyntaxError(msg, 0)

    def load_mcode_file(self, filename):
        """Open a file with the machine code. Parse it. Returns the a list with
           all the machine codes"""

        Lexer.COMMENT_SYMBOL = "//"

        # - Start loading from the addr 0
        self._load_addr = 0

        # -- Read the file
        try:
            with open(filename, mode='r') as f:
                raw = f.read()
        except:
            print("Error: file not found: {}".format(filename))
            sys.exit()

        # REGEX_MCODE = r"[0-9a-fA-F]+"

        # -- Split the ASCII file into isolates lines
        raw = raw.splitlines()
        for line in raw:
            # - Remove blank lines
            if Lexer.is_blank_line(line) or Lexer.is_comment_line(line):
                continue

            try:
                self._parse_mcode(line)
            except SyntaxError as e:
                print(e.msg)
                sys.exit()

        print("\nprog.list loaded into memory!")

    def _code2asm(self, inst):
        """Return a string with the given machine code instruction
           in assembly language"""

        opcode, addr = self._decode(inst)
        nemonic = self.NEMONIC[opcode]
        asm = "{}".format(nemonic)

        # - Add the argument
        if nemonic in ["ST", "LD", "ADD", "BR", "BZ"]:
            asm += " /H'{:03X}".format(addr)

        # - Add the content of the memory (as a comment)
        if nemonic in ["ST", "LD", "ADD"]:
            asm += ";  [{:03X}]={:03X}".format(addr, self._mem[addr])

        return asm


def example_simplez2(s):
    """Example of simulation of the program SIMPLEZ2"""

    # - Load the machine code
    # s.load(Test_progs.SIMPLEZ2)

    # - Add the variables to watch (label - address)
    s.add_var("cont", 46)
    s.add_var("pen", 47)
    s.add_var("ult", 48)
    s.add_var("sig", 49)
    s.add_var("SUM", 50)


def example_simplez3(s):
    s.add_var("SUM", 150)


def example_simplez4(s):
    s.add_var("cien", 100)
    s.add_var("doscientas", 200)
    s.add_var("cont", 3)
    s.add_var("temp", 4)


def example_simplez5(s):
    s.add_var("res1", 52)
    s.add_var("res2", 102)
    s.add_var("resta", 201)
    s.add_var("arg", 202)


if __name__ == "__main__":
    """Main program"""

    # - Create the virtual simplez processor
    s = simplez()

    # -- Load the machine code to simulate
    s.load_mcode_file("prog.list")

    # -- Simulate the SIMPLEZ2 example
    # example_simplez5(s)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# This file is part of the VPM Project

"""
Toolchain-simplez are a set of tools for working with the education processor
Simplez. The included tools are:

* sasm: Simplez Assembler
* ssim: Simplez simulator
* sreset: Simplez Reset
* sboot: Simplez loader (a bootloader is needed)
"""

__author__ = 'Juan Gonzalez-Gomez (obijuan)'
__email__ = 'juan.gonzalez@bq.com'
__license__ = 'GPLv2'
__version__ = '0.0.11'



#! /usr/bin/python3
# ---------------------------------------------------------------------------
# --  (C)2002-2004 Chris Liechti <cliecht@gmx.net>
# --  (C) 2007 Juan Gonzalez
# --  (C) 2016 Juan Gonzalez
# --
# --  Modulo para trabajar con el teclado de la consola sin tener que
# --  "apretar" enter. Es multiplataforma, para Linux y Windows
# --
# --  LICENCIA GPL
# ---------------------------------------------------------------------------

import os

# -- Seleccionar la plataforma

# -- Windows
if os.name == 'nt':
    import msvcrt

    def getkey():
        while 1:
            z = msvcrt.getch()
            if z == '\0' or z == '\xe0':    # functions keys
                msvcrt.getch()
            else:
                if z == '\r':
                    return '\n'
                return z

# -- Sistemas Posix
elif os.name == 'posix':

    import termios
    import sys
    import atexit

    # -- Abrir la consola y modificar sus propiedades
    fd = sys.stdin.fileno()
    old = termios.tcgetattr(fd)
    new = termios.tcgetattr(fd)
    new[3] = new[3] & ~termios.ICANON & ~termios.ECHO
    new[6][termios.VMIN] = 1
    new[6][termios.VTIME] = 0

    # -- Cambiar los atributos
    termios.tcsetattr(fd, termios.TCSANOW, new)

    # -- Funcion para leer una tecla
    def getkey():
        c = os.read(fd, 1)
        return c

    # -- Funcion para restablecer la consola al finalizar
    def cleanup_console():
        termios.tcsetattr(fd, termios.TCSAFLUSH, old)

    # -- Restaurar el terminal a la salida
    atexit.register(cleanup_console)

else:
    raise "Consola_io No esta implementado para la plataforma".format(sys.platform)

# -------------------------------------
# Pequena prueba del modulo
# -------------------------------------
if __name__ == '__main__':

    print ("Pulse una tecla")
    getkey()
    print ("Adios...")



#!/usr/bin/python3

import ply.lex as lex


class Lexer(object):

    tokens = (
       'COMMENT',
       'ADDR',      # -- Value: addr (in hexa)
       'DATA',      # -- Value: Data (in hexa)
       'EOF'
    )

    t_ignore = ' \t\r\f\v'

    def __init__(self, data):
        """Create the lexer and give the data"""
        self.lexer = lex.lex(module=self)
        self.lexer.input(data)

        # -- Read the first token
        self.current_token = self.lexer.token()

        # -- Address pointer. It contains the current address
        self.addr = 0

    # - Comments are ignored
    def t_COMMENT(self, t):
        r'//[^\n]*'
        pass

    def t_DATA(self, t):
        r'[0-9a-fA-F]+'
        t.value = int(t.value, 16)
        return t

    def t_ADDR(self, t):
        r'@[0-9a-fA-F]+'
        t.value = int(t.value[1:], 16)
        return t

    def t_newline(self, t):
        r'\n+'
        t.lexer.lineno += len(t.value)

    def t_eof(self, t):
        t.type = 'EOF'
        t.value = ''
        return t

    # Error handling rule
    def t_error(self, t):
        print("Illegal character '%s'" % t.value[0])
        t.lexer.skip(1)

    def token(self):
        """Read the next token"""
        self.current_token = self.lexer.token()

    def get_block(self):
        """Return the next memory block. It returns:
        (initial address, memory block)"""

        # -- Initial address = current address
        init_addr = self.addr

        # -- Current memory block
        bmem = []

        if self.current_token.type == "EOF":
            return None, None

        if self.current_token.type == "ADDR":

            # -- Updte the initial address
            init_addr = self.current_token.value

            # -- Read the next token
            self.token()

        while self.current_token.type == "DATA":
            # -- Add the data to the memory block
            bmem.append(self.current_token.value)

            self.token()

        return init_addr, bmem

# -- Main program
if __name__ == '__main__':

    data = '''
    //-- Hola
    @10
    20
    @40  //-- Comienzo
    10 20   //-- Una inst
    30   //-- Otra...
    caca
    @50
    '''

    data2 = '''
    @10
    '''
    data3 = ''' '''

    # Create the lexer with some data
    l = Lexer(data)

    # -- Read the first block
    init_addr, bmem = l.get_block()
    i = 1

    # -- Read every block and print its information
    while init_addr:
        print("Block {}".format(i))
        print("  Addr: {:03X}".format(init_addr))
        print("  Size: {}".format(len(bmem)))
        print("  Mem: {}".format(bmem))
        print()

        init_addr, bmem = l.get_block()
        i += 1



#!/usr/bin/python3
# ------------------------------------------------------------------------------
# -  Cargador de programas en SIMPLEZ
# --  (C) BQ Enero 2016. Written by Juan Gonzalez (obijuan)
# --  Python 3
# ------------------------------------------------------------------------------
# - La FPGA tiene que estar configurada con SIMPLEZ con el programa boot.asm
# grabado (es un bootloader).
# - sboot.py se comunica con el bootloader
# - Al arrancar SIMPLEZ se ejecuta el bootloader. El protocolo es el siguiente:
# -
# -  Bootloader: Envía el caracter BREADY para indicar que esta listo
# -  sboot envía primero el tamaño del fichero a cargar, en palabras
# -  Cada palabra se envía como 2 bytes, primero el de mayor peso y luego
# -  el de menor
# -  A continuación sboot comienza a enviar todas las palabras que componen
# - el programa
# -  Cuando se han enviado todas, el bootloader le pasa el control al nuevo
# - programa cargado
# ------------------------------------------------------------------------------
# - Los programas a cargar tiene que comenzar a patir de la direccion 40h
# ------------------------------------------------------------------------------

import serial
import time
import sys
import vmem
import consola_io
import threading


# -- Character for quiting the interactive mode
EXITCHAR = b'\x04'

# - Example programs. For testing
LEDS = [0x303, 0x1FB, 0xE00, 0x00E]
SEC1 = [0x307, 0x1FB, 0xF00, 0x308, 0x1FB, 0xF00, 0x700, 0x009, 0x006]
SEC2 = [0x307, 0x1FB, 0xF00, 0x308, 0x1FB, 0xF00, 0x700, 0x00C, 0x003]

# -- Byte that is received when the bootloader is ready
BREADY = b'B'

# -- Initial address were the programs are loaded
INITIAL_ADDR = 0x40

# -- Flag for indicating that the threads are executing
executing = 1


# -- Download the program
def download(ser, prog):

    tam = len(prog)

    # -- Send program size (in words)
    tamb = tam.to_bytes(2, byteorder='big')
    ser.write(tamb)

    # -- Transmit the program
    for inst in prog:

        # -- Convert the word to bytes
        instb = inst.to_bytes(2, byteorder='big')

        # -- Send the bytes
        ser.write(instb)


def reader(ser):
    """Thread for reading data from simplez and printing on the screen"""
    while executing:
        try:
            data = ser.read()
        except:
            print("Serial reading error")
            sys.exit(0)

        try:
            sys.stdout.write(data.decode("utf-8"))
            sys.stdout.flush()
        except:
            pass


def parse_file(filename):
    """Parse the file with the machine code. It returns:
    (Initial address and memory block)"""

    # -- Read the file
    try:
        with open(filename, mode='r') as f:
            rawdata = f.read()
    except:
        print("Error: file not found: {}".format(filename))
        sys.exit()

    # Create the lexer with some data
    l = vmem.Lexer(rawdata)

    # -- Read the first block
    init_addr, bmem = l.get_block()

    # -- Return the size, initial addr and memory block
    return init_addr, bmem


def parse_arguments():
    """Parse the arguments"""

    import argparse
    description = \
        """SIMPLEZ Loader.  Load programs into the simplez RAM memory"""

    # -- Add the description
    parser = argparse.ArgumentParser(description=description)

    # -- Add the assembler input file
    parser.add_argument("inputfile", help="Simplez machine code file (.list)")

    parser.add_argument("-t", help="Load a test example (-t 1)",
                        action="store_true")

    parser.add_argument(
        "-i",
        help="Open an interactive serial terminal for comunicating with Simplez",
        action="store_true")

    # -- Parse the anguments
    args = parser.parse_args()

    # -- Return the input file, test and interactive
    return args.inputfile, args.t, args.i


def main():

    # -- Process the arguments
    input_file, test, interactive = parse_arguments()

    if test:
        # -- Load the test example
        print("Test!")
        prog = [0x24D, 0x1FB, 0xF00, 0x24E, 0x1FB, 0xF00, 0x24F, 0x1FB, 0xF00,
                0x250, 0x1FB, 0xF00, 0x640, 0x001, 0x002, 0x004, 0x008]
    else:
        # -- Load from file
        # -- Parse the input file. Format: verilog memory. Data is hexadecimal
        init_addr, prog = parse_file(input_file)

        print("File: {}".format(input_file))
        print("Size: {} words".format(len(prog)))
        print("Initial address: H'{:03X}".format(init_addr))

        if init_addr < INITIAL_ADDR:
            print("Error: Initial address below H'{:03X}".format(INITIAL_ADDR))
            sys.exit(0)

        if init_addr > INITIAL_ADDR:
            print("Warning: Initial address is NOT H'{:03X}".format(
                   INITIAL_ADDR))

    ser = serial.Serial('/dev/ttyUSB1', baudrate=115200, timeout=0.5)

    # -- Reset Simplez
    ser.setDTR(1)
    time.sleep(0.2)
    ser.setDTR(0)

    # -- Wait for the character "B" to be received. It means the bootloader
    # --  is ready
    if ser.read() == BREADY:
        print("Bootloader ready!!!!")
    else:
        print("ERROR: NO bootloader")
        sys.exit(0)

    # download(ser, LEDS)
    # download(ser, SEC2)
    download(ser, prog)

    print("EXECUTING!!!")

    # -- If in interactive mode (-i option), a simple terminal is created
    if interactive:
        print("Entering the interactive mode...")
        print("Press CTRL-D to exit\n")

        # -- Launch a thread for reading the data coming from simplez
        r = threading.Thread(target=reader, args=[ser])
        r.start()

        # -- Sending data to simplez from the keyboard
        while 1:
            try:
                # -- Wait for a key typed
                c = consola_io.getkey()

                # -- Exit char
                if c == EXITCHAR:
                    break
                else:
                    # -- Send the char to simplez
                    ser.write(c)

            except:  # -- Si se ha pulsado control-c terminar
                print ("Abortando...")
                break

        global executing
        executing = 0
        r.join()

    ser.close()

# -- Main program
if __name__ == '__main__':
    main()



#!/usr/bin/env python3

from setuptools import setup

import toolchain_simplez

setup(name='toolchain-simplez',
      version=toolchain_simplez.__version__,
      description=toolchain_simplez.__doc__,
      author=toolchain_simplez.__author__,
      author_email=toolchain_simplez.__email__,
      url='https://github.com/Obijuan/simplez-fpga',
      download_url='https://pypi.python.org/pypi/toolchain-simplez',
      license=toolchain_simplez.__license__,
      py_modules=['toolchain_simplez', 'sasm', 'sreset', 'sboot', 'vmem',
                  'consola_io', 'ssim'],
      entry_points={'console_scripts': ['sasm=sasm:main',
                                        'sreset=sreset:main',
                                        'sboot=sboot:main']},
      install_requires=['pyserial', 'ply'],
      classifiers=[
          'Development Status :: 1 - Planning',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Programming Language :: Python :: 3.4'
      ])



#!/usr/bin/python3
# ------------------------------------------------------------------------------
# --  Asembler for the Simplez microprocessor
# --  (C) BQ November 2015. Written by Juan Gonzalez (obijuan)
# --  Python 3
# --  v1.2
# ------------------------------------------------------------------------------
# -- Released under the GPL v3 License
# ------------------------------------------------------------------------------
# - Grammar definition
# -
# --- A program is a list of lines with the END keyword in the end
#     (finished by EOL)
# --- Notice that after the END there could be more instructions, but
#     they will be ignored
# <program> ::= <lines> "END" EOL
#
# --- There could be more than one lines. Each line should be end by EOL
# <lines> ::= (<line> EOL+ )* | <lines>
#
# -- Each line can be a comment, a line of code followed by a commnet
# -- or just a simple line of code
# <line> ::= COMMENT | <lineofcode> COMMENT | <lineofcode>
#
# -- Lines of code can be either a directive or line with a simplez instructions
# <lineofcode> ::= <directive> | <lineinstruction>
#
# -- All the instruction can have a label in the beginning of the line
#   (optionally)
# <lineinstruction> ::= <instruction> | LABEL <instruction>
#
# -- There are 9 different simplez-F instructions
# <instruction> ::= <instLD>  | <insST>   | <instADD>  | <instBR> | <instBZ> |
# ----------------- <instCLR> | <instDEC> | <instHALT> | <instWAIT>
#

# - There are 4 different directives
# <directive> ::= <dirORG> | <dirEQU> | <dirRES> | <dirDATA>
#
# - ORG directive do not have any label in the left
#   The argument can be a number or a label
# - label (defined by the EQU directive)
# <dirORG> ::= ORG NUMBER | ORG LABEL
#
# -- EQU Directive
# <dirEQU> ::= LABEL EQU NUMBER
#
# -- DATA directive.  Label is optional
# <dirDATA> ::= LABEL DATA <datacollection> |  DATA <datacollection>
#
# -- Collection of data,separated by ,
# <datacollection> ::= <data> (,<data>)*
#
# -- Type of data accepted as "DATA"
# <data> ::= STRING | NUMBER
#
# -- RES directive. Label is optional
# <dirRES> ::= LABEL RES NUMBER | RES NUMBER
#

# -- Instruction LD
# <instLD> ::= LD <addr>
#
# -- The address can be giben numerically (eg. /501), or by label (eg. /ini)
# <addr> ::= ADDRNUM | ADDRLABEL
#
# -- Instruction ST
# <instST> ::= ST <addr>
#
# -- Instruction ADD
# <instADD> ::= ADD <addr>
#
# -- Instruction BR
# <instBR> ::= BR <addr>
#
# -- Instruction BZ
# <instrBZ> ::= BZ <addr>
#
# -- Instruction CLR
# <instrCLR> ::= CLR
#
# -- Instruction DEC
# <instrDEC> ::= DEC
#
# -- Istruction HALT
# <instrHAL> ::= HALT
#
# -- Instruction WAIT
# <instWAIT> ::= WAIT

# - Tokens:
#
# - EOL, EOF, COMMENT, LABEL, ORG, NUMBER, STRING, ADDRNUM, ADDRLABEL
# - LD, ST, ADD, BR, BZ, CLR, DEC, HALT, WAIT

# - COMMENT: ;(any ascii char)*
# - LABEL: (any ascii char)*  That is NOT a reserved word
# - NUMBER: Decimal or hexadecimal number:  [0-9]* | H'[0-9,a-f,A-F]*
# - STRING: "(any ascii char)"
# - ADDRNUM: /NUMBER
# - ADDRLABEL: /LABEL

import sys
import re

# --- Token types
(EOL, EOF, COMMENT, LABEL, ORG, NUMBER, ADDR,
 LD, ST, ADD, BR, BZ, CLR, DEC, HALT, WAIT, UNKNOWN, END, EQU, RES,
 DATA, STRING, COMMA) = (
 'EOL', 'EOF', 'COMMENT', 'LABEL', 'ORG', 'NUMBER',  'ADDR',
 'LD', 'ST', 'ADD', 'BR', 'BZ', 'CLR', 'DEC', 'HALT', 'WAIT', 'UNKNOWN', 'END',
 'EQU', 'RES', 'DATA', 'STRING', 'COMMA'
)

# -- default output file with the machine code for SIMPLEZ
OUTPUT_FILE = "prog.list"

VERSION = "v1.2"

# -- Instruction opcodes
OPCODES = {ST: 0, LD: 1, ADD: 2, BR: 3, BZ: 4,
           CLR: 5, DEC: 6, HALT: 7, WAIT: 0xF, DATA: 0xFF, }


class Token(object):
    """Token generator"""

    def __init__(self, type, value, line=None):
        self.type = type
        self.value = value
        self.line = line

    def __str__(self):

        # -- Print the Token line number
        string = "[{}] Token: ".format(self.line)

        # -- Case 1: These token have no value
        if self.type in [EOL, EOF, END, ORG, EQU, RES, DATA,
                         COMMA, LD, ST, ADD, BR, BZ, CLR, DEC, HALT, WAIT]:
            string += self.type

        # -- Case 2: These tokens have values
        elif self.type in [COMMENT, STRING, NUMBER, LABEL, ADDR]:
            string += "{} ({})".format(self.type, self.value)

        else:
            string += "Unknown ({})".format(self.value)

        return string

# ------------- Regular expresion definitions for the Lexer --------
# -- Decimal number
REGEX_DEC = r"[0-9]+"

# -- Hexadecimal number
REGEX_HEX = r"H\'[0-9a-fA-F]+"

# -- Comment
REGEX_COMMENT = r";[^\n]*"

# -- White spaces. \n is not included
REGEX_WSPACE = r"[ \t\r\f\v]+"

# -- Label
REGEX_LABEL = r"[a-zA-Z0-9_]+"

# -- String
REGEX_STRING = r"\"[^\"]*\""

# -- Numeric address (in decimal)
REGEX_ADDRNUM1 = r"/[0-9]+"

# -- Numeric address (in hexadecimal)
REGEX_ADDRNUM2 = r"/H\'[0-9a-fA-F]+"

# -- Address label
REGEX_ADDRLABEL = r"/[a-zA-Z0-9_]+"


class Lexer(object):
    """Lexical analyzer"""

    def __init__(self, text):
        self.text = text
        self.pos = 0
        self.line = 1

    def reset(self):
        self.pos = 0
        self.line = 1

    def check_hexnumber(self):
        """Check if it is an hexadecimal number. If so, return its value,
           else None
        """
        scan = re.match(REGEX_HEX, self.text[self.pos:].upper())
        if scan:
            self.pos += len(scan.group())

            # -- Return the number
            return int(scan.group()[2:], 16)
        else:
            return None

    def check_decimal(self):
        """Check if it is a decimal number. If so, return it svalue,
           else None"""

        scan = re.match(REGEX_DEC, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return int(scan.group())

    def check_comment(self):
        """Check if it is a comment. It returns the comment or None"""

        scan = re.match(REGEX_COMMENT, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return scan.group()[1:]

    def check_addr(self):
        """Check if it is an address"""

        # -- Case 1: Address in decimal
        scan = re.match(REGEX_ADDRNUM1, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return int(scan.group()[1:])

        # -- Case 2: Address in hexadecimal
        scan = re.match(REGEX_ADDRNUM2, self.text[self.pos:].upper())
        if scan:
            self.pos += len(scan.group())
            return int(scan.group()[3:], 16)

        # -- Case 3: Address is a label
        scan = re.match(REGEX_ADDRLABEL, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return scan.group()[1:]

    def check_directive(self):
        """Check if it is a directive"""

        for direct in [END, ORG, EQU, RES, DATA]:
            scan = re.match(direct+"\s", self.text[self.pos:].upper())
            if scan:
                self.pos += len(scan.group()[:-1])
                return direct

    def check_instruction(self):
        """Check if it is an instruction"""

        for instr in [LD, ST, ADD, BR, BZ, CLR, DEC, HALT, WAIT]:
            scan = re.match(instr+"\s", self.text[self.pos:].upper())
            if scan:
                self.pos += len(scan.group()[:-1])
                return instr

    def check_label(self):
        """Check if it is a label"""

        scan = re.match(REGEX_LABEL, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return scan.group()

    def check_string(self):
        """Check if it is a string"""
        scan = re.match(REGEX_STRING, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())
            return scan.group()[1:-1]

    def get_token(self):
        """Get the next token from tex"""

        # -- Remove the white spaces (except \n)
        scan = re.match(REGEX_WSPACE, self.text[self.pos:])
        if scan:
            self.pos += len(scan.group())

        # -- Is it a commnet?
        comment = self.check_comment()
        if comment:
            return Token(COMMENT, comment, line=self.line)

        # -- Is it a hexadecimal number?
        hexnumber = self.check_hexnumber()
        if hexnumber is not None:
            return Token(NUMBER, hexnumber, line=self.line)

        # -- Is it a decimal number?
        number = self.check_decimal()
        if number is not None:
            return Token(NUMBER, number, line=self.line)

        # -- Is it a string?
        string = self.check_string()
        if string is not None:
            return Token(STRING, string, line=self.line)

        # -- Is it an address?
        addr = self.check_addr()
        if addr is not None:
            return Token(ADDR, addr, line=self.line)

        # -- Check if it is a directive
        direct = self.check_directive()
        if direct:
            return Token(direct, None, line=self.line)

        # -- Check if it is an instruction
        instr = self.check_instruction()
        if instr:
            return Token(instr, None, line=self.line)

        # -- Check if it is a label
        label = self.check_label()
        if label:
            return Token(LABEL, label, line=self.line)

        # --Get the current char
        try:
            current_char = self.text[self.pos]
        except IndexError:
            return Token(EOF, None, line=self.line)

        # -- Is it a EOL?
        if current_char == '\n':
            self.pos += 1
            line = self.line
            self.line += 1
            return Token(EOL, None, line=line)

        # -- Is it a COMMA?
        if current_char == ',':
            self.pos += 1
            return Token(COMMA, None, line=self.line)

        self.pos += 1
        return Token(UNKNOWN, current_char, line=self.line)

    def test(self):
        """Test the lexer"""

        string = ""
        while True:
            token = self.get_token()
            string += "{}\n".format(token)
            if token.type == EOF:
                return string


# ---------------------- AST
class Prog_AST(object):
    def __init__(self):
        self.addr = 0   # -- Current address
        self.linst = []  # -- List of instructions

        # -- Symbol table. It is used for storing the pairs label - address
        self.symtable = {}

    def add(self, inst):
        """Add the instruction in the current address.
           Increment the address counter
        """

        # -- Assign the current address
        inst.addr = self.addr

        # -- Insert the instruction
        self.linst.append(inst)

        # -- Increment the current address
        self.addr += 1

    def solve_labels(self):
        """semantic analysis. Assign a value to the instruction arguments
           that do not have a numeric value"""

        for instr in self.linst:

            # -- Check only the instructions with arguments
            if instr.arg is not None:
                # -- Check if the argument is a string
                if type(instr.arg) is str:
                    # -- It is a label. Put the label in the label attribute
                    instr.label = instr.arg

                    # -- Get the label's value from the symtable
                    try:
                        value = self.symtable[instr.label]
                    except KeyError:
                        raise Exception(
                            "Error: Line: {}: Symbol not defined: {}".format(
                                instr.line,
                                instr.label))

                    # -- Write the value in the argument
                    instr.arg = int(value)

    def assembly(self):
        """Write the program in assembly language"""
        string = ""
        addr = 0
        for instr in self.linst:
            # -- There is a gap between in the addresses
            if addr != instr.addr:
                string += "\n      ORG 0x{0:03X}\n".format(instr.addr)
                addr = instr.addr

            string += "{}\n".format(instr)
            addr += 1

        return string

    def size(self):
        """Calculate the program size (code, data), in words
         """

        # -- Calculate the size of data
        # -- Data above the address 504 is not counted
        sdata = 0
        scode = 0
        for instr in self.linst:
            # -- Only the data / code below 504 counts
            if instr.addr < 504:
                if instr.nemonic == DATA:
                    sdata += 1
                else:
                    scode += 1

        return scode, sdata

    def show_symbols(self):
        """Print the symbol table"""

        for key, value in self.symtable.items():
            print("{} = {}".format(key, value))

    def machine_code(self, asm=False):
        """Write the program in machine code"""
        string = ""
        if asm:
            string += "//mcode    addr nemonic\n"
            string += "//-----    ---- -------\n"

        addr = 0
        for instr in self.linst:
            if addr != instr.addr:

                if asm:
                    string += "\n"
                string += "@{0:03X}".format(instr.addr)
                if asm:
                    string += "  //--       ORG {0:03X}".format(instr.addr)

                string += "\n"
                addr = instr.addr

            string += "{:03X}".format(instr.mcode())
            if asm:
                string += "   //-- {}".format(instr)
            string += "\n"
            addr += 1

        return string


class Instruction(object):
    """Simplez instruction class"""

    def __init__(self, nemonic, arg=None, line=None):
        self.nemonic = nemonic  # -- Instruction name
        self.arg = arg          # -- Instruction argument
        self.line = line        # -- src line number where the inst. is located
        self.addr = None        # -- Address were the instruction is stored
        self.label = None       # -- If the argument is a label

    def opcode(self):
        """Return the instruction opcode"""
        return OPCODES[self.nemonic]

    def mcode(self):
        """Return the machine code"""
        if self.nemonic == "DATA":
            return self.arg
        elif self.nemonic == "WAIT":
            return 0xF00
        elif self.arg is not None:
            return (self.opcode() << 9) + self.arg
        else:
            return (self.opcode() << 9)

    def __str__(self):
        string = ""

        if self.addr is not None:
            string += "[{:03X}] ".format(self.addr)

        # -- Uncomment for showing the src line numbers
        # if self.line is not None:
        #     string += "(Line: {}) ".format(self.line)

        string += "{}".format(self.nemonic)

        if self.arg is not None:
            if self.nemonic == DATA:
                string += " {}".format(self.arg)
            else:
                if type(self.arg) is int:
                    string += " /{:03X}".format(self.arg)
                else:
                    string += " /{}".format(self.arg)

        return string

# ----------- Syntax analyzer


class Parser(object):
    """Sintax analysis"""

    def __init__(self, lexer):
        self.lexer = lexer
        self.prog = Prog_AST()

        # -- The last two token are read in advance
        self.current_token = self.lexer.get_token()
        self.next_token = self.lexer.get_token()

    def error(self, msg=None, line=None):
        raise Exception('Error: {}. Line: {}'.format(msg, line))

    def assert_type(self, type, error_msg="", debug=False):
        """Make sure the current token is of the given type"""

        if self.current_token.type == type:
            if debug:
                print(self.current_token)

            self.current_token = self.next_token
            self.next_token = self.lexer.get_token()

        else:
            self.error(error_msg, line=self.current_token.line)

    def program(self):
        """<program> ::= <lines> "END" EOL"""

        self.lines()
        self.assert_type(END, "END expected")
        self.assert_type(EOL, "No EOL after END")

    def lines(self):
        """<lines> ::= EOL*  (<line> EOL+)* """

        while self.current_token.type not in [EOF, END]:

            self.line()
            self.assert_type(EOL, "Unexpected element")

            # -- Remove the extra EOL (if any)
            while (self.current_token.type == EOL):
                self.assert_type(EOL)

    def line(self):
        """<line> ::= COMMENT | <lineofcode> (COMMENT)"""

        if self.current_token.type == COMMENT:
            self.assert_type(COMMENT)

        else:
            if self.lineofcode():
                if (self.current_token.type == COMMENT):
                    self.assert_type(COMMENT)

    def lineofcode(self):
        """<lineofcode> ::= <directive> | <lineinstruction>"""

        if self.directive():
            return True
        else:
            # -- It is not a directive, should be a instruction
            return self.lineinstruction()

    def directive(self):
        """<directive> ::= <dirORG> | <dirEQU> | <dirRES> | <dirDATA>"""

        if self.dir_org():
            return True
        elif self.dir_equ():
            return True
        elif self.dir_res():
            return True
        elif self.dir_data():
            return True
        else:
            # -- It is not a directive
            return False

    def dir_org(self):
        """<dirORG> ::= ORG NUMBER | ORG LABEL"""

        # -- If it is not an ORG token, return
        if self.current_token.type == ORG:
            self.assert_type(ORG)

            if self.current_token.type == NUMBER:
                addr = self.current_token.value
                self.assert_type(NUMBER)

                # -- Change the current address
                self.prog.addr = addr

            # -- If the address is not a number... it should be a label
            else:
                label = self.current_token.value
                line = self.current_token.line
                self.assert_type(LABEL, "ORG: Label or number expected")

                # -- Get the address asociated to the label
                try:
                    addr = self.prog.symtable[label]
                except KeyError:
                    self.error("Unknow Label: {}".format(label), line=line)

                # -- Change the current address
                self.prog.addr = addr

            return True

        # -- Not an ORG directive
        else:
            return False

    def dir_equ(self):
        """<dirEQU> ::= LABEL EQU NUMBER"""

        if self.current_token.type == LABEL and self.next_token.type == EQU:
            label = self.current_token.value
            line = self.current_token.line
            self.assert_type(LABEL)
            self.assert_type(EQU)
            value = self.current_token.value
            self.assert_type(NUMBER, "EQU: Expected a number")

            # - check if the label is already in the table
            if label in self.prog.symtable:
                self.error(msg="Duplicated label: {}".format(label), line=line)

            # - Insert the label in the symbol table
            self.prog.symtable[label] = value

            return True
        elif self.current_token.type == EQU:
            self.error("EQU without label", line=self.current_token.line)
        else:
            # -- It is not an EQU directive
            return False

    def dir_res(self):
        """<dirRES> ::= (LABEL) RES NUMBER"""

        # -- Case 1:  LABEL RES
        if self.current_token.type == LABEL and self.next_token.type == RES:
            label = self.current_token.value
            line = self.current_token.line
            self.assert_type(LABEL)

            # - check if the label is already in the table
            if label in self.prog.symtable:
                self.error(msg="Duplicated label: {}".format(label), line=line)

            # - Insert the label in the symbol table
            self.prog.symtable[label] = self.prog.addr

        # -- Case 2: RES (without label)
        if self.current_token.type == RES:
            line = self.current_token.line
            self.assert_type(RES)
            value = self.current_token.value
            self.assert_type(NUMBER)

            # -- Reserving 0 words does not make any sense
            if value == 0:
                self.error(msg="Not possible to reserve 0 words", line=line)

            # - Create all the data instructions reserved by the directive res
            for i in range(value):
                instr = Instruction(DATA, line=line, arg=0)
                self.prog.add(instr)

            return True
        else:
            return False

    def dir_data(self):
        """<dirDATA> ::= LABEL DATA <datacollection> |  DATA <datacollection>"""

        # -- Case 1: LABEL DATA
        if self.current_token.type == LABEL and self.next_token.type == DATA:

            # -- Get the label
            label = self.current_token.value
            line = self.current_token.line
            self.assert_type(LABEL)

            # - check if the label is already in the table
            if label in self.prog.symtable:
                self.error(msg="Duplicated label: {}".format(label), line=line)

            # -- Insert the lable in the symbol table
            self.prog.symtable[label] = self.prog.addr

            self.assert_type(DATA)
            self.data_collection()

            return True

        # -- Case 2: DATA (no label)
        if self.current_token.type == DATA:
            self.assert_type(DATA)
            self.data_collection()

            return True

        # -- It is not a DATA directive
        return False

    def data_collection(self):
        """<datacollection> ::= <data> (,<data>)*"""

        self.data()
        while self.current_token.type == COMMA:
            self.assert_type(COMMA)
            self.data()

    def data(self):
        """<data> ::= STRING | NUMBER | LABEL"""

        line = self.current_token.line
        value = self.current_token.value

        if self.current_token.type == NUMBER:
            self.assert_type(NUMBER)
            instr = Instruction(DATA, line=line, arg=value)
            self.prog.add(instr)
        elif self.current_token.type == STRING:
            self.assert_type(STRING)

            # -- Insert one data instruction per character in the string
            for char in value:
                instr = Instruction(DATA, line=line, arg=ord(char))
                self.prog.add(instr)
        elif self.current_token.type == LABEL:
            self.assert_type(LABEL)
            instr = Instruction(DATA, line=line, arg=value)
            self.prog.add(instr)

    def lineinstruction(self):
        """<lineinstruction> ::= (LABEL) <instruction>"""

        if self.current_token.type == LABEL:

            line = self.current_token.line
            label = self.current_token.value

            # - check if the label is already in the table
            if label in self.prog.symtable:
                self.error(msg="Duplicated label: {}".format(label), line=line)

            # - Insert the label in the symbol table
            self.prog.symtable[label] = self.prog.addr

            self.assert_type(LABEL)

            # -- If there is a label, there should be an instrucction
            # -- in the same line
            if self.instruction():
                return True
            else:
                self.error("Invalid instruction", line=line)
                return False

        # -- There should be now an instruction
        line = self.current_token.line
        self.instruction()
        return True

    def instruction(self):
        """<instruction> ::= <instLD>  | <insST>   | <instADD>  | <instBR> |
                             <instBZ>  | <instCLR> | <instDEC> | <instHALT> |
                             <instWAIT>"""

        # -- Case 1: Instructions with no arguments
        for instr in [CLR, DEC, HALT, WAIT]:
            if self.parse_instr0(instr):
                return True

        # -- Case 2: Instructions with 1 argument (an absolute address)
        for instr in [ST, LD, ADD, BR, BZ]:
            if self.parse_instr1(instr):
                return True

        # -- It is not an instruction
        return False

    def parse_instr0(self, inst_type):
        """Parse the instructions with 0 arguments
           HALT, WAIT, DEC, CLR
        """

        if self.current_token.type == inst_type:
            line = self.current_token.line
            self.assert_type(inst_type)
            instr = Instruction(inst_type, line=line)
            self.prog.add(instr)
            return True
        else:
            return False

    def parse_instr1(self, inst_type):
        """Parse the instructions with 1 argument
           ST, LD, ADD, BR, BZ
        """

        if self.current_token.type == inst_type:
            line = self.current_token.line
            self.assert_type(inst_type)
            arg = self.current_token.value
            self.assert_type(ADDR, "Invalid address")
            instr = Instruction(inst_type, arg=arg, line=line)
            self.prog.add(instr)
            return True
        else:
            return False

    def parse(self):
        self.program()
        return self.prog


def parse_arguments():
    """Parse the arguments, open the asm file and return the raw contents"""

    import argparse
    description = """
        Simplez assembler. The prog.list file with the machine code is \
        generated as output
    """
    # -- Add the assembler description
    parser = argparse.ArgumentParser(description=description)

    # -- Add the assembler argument: asmfile
    parser.add_argument("asmfile", help="Simplez asembly file (.asm)")

    # -- Add the assembler argument: verbose
    parser.add_argument("-verbose", help="verbose mode on", action="store_true")

    # -- Output file argument
    parser.add_argument("-o", metavar="output", default=OUTPUT_FILE,
                        help="Specify the output filename")

    # -- Parse the anguments
    args = parser.parse_args()

    # -- File to assembly
    asmfile = args.asmfile

    # -- Output file
    output_file = args.o

    # -- Read the file
    try:
        with open(asmfile, mode='r') as f:
            raw = f.read()
    except:
        print("Error: file not found: {}".format(asmfile))
        sys.exit()

    # -- Return the file and verbose arguments
    return raw, asmfile, output_file, args.verbose


def main():

    # -- Process the arguments. Return the source file and the verbose flags
    asmfile, filename, output_file, verbose = parse_arguments()

    print("Assembler for the SIMPLEZ microprocessor (Version: {})".format(
        VERSION))

    print("Released under the GPL license\n")

    # -- Lexical analysis
    lexer = Lexer(asmfile)

    if verbose:
        print("\n------- Lexical analysis")

        lexer_log = lexer.test()
        print(lexer_log)
        lexer.reset()

        print("\n------- Sintax Analysis ------")

    # -- Syntax analysis
    parser = Parser(lexer)
    try:
        prog = parser.parse()
    except Exception as inst:
        print(inst)
        sys.exit(0)

    if verbose:
        print()
        asmcode1 = prog.assembly()
        print(asmcode1)
        print("Symbols:")
        prog.show_symbols()

        print()
        print("-------- Semantics analysis:")

    try:
        prog.solve_labels()
    except Exception as inst:
        print(inst)
        sys.exit(0)

    if verbose:
        asmcode2 = prog.assembly()
        print(asmcode2)

        print("-------- Generated machine code")

    mcode = ""
    mcode += "//-- Source file: {}\n".format(filename)
    mcode += "//-- Output file format: verilog\n\n"
    mcode += prog.machine_code(asm=True)

    if verbose:
        print(mcode)

    # -- Write the machine code in the output file file
    with open(output_file, mode='w') as f:
        f.write(mcode)

    print("OK! Machine code for SIMPLEZ generated\n")
    print("File: {}".format(output_file))
    scode, sdata = prog.size()
    print("Size:   {} words".format(scode + sdata))
    print("  Code: {} words".format(scode))
    print("  data: {} words".format(sdata))

# -- Main program
if __name__ == '__main__':
    main()



#!/usr/bin/python3

import serial
import time


def main():
    ser = serial.Serial('/dev/ttyUSB1', baudrate=115200)

    # -- Reset Simplez
    ser.setDTR(1)
    time.sleep(0.2)
    ser.setDTR(0)

# -- Main program
if __name__ == '__main__':
    main()
